<?php
/**
 * @package     CSVI
 * @subpackage  Form2Content
 *
 * @author      RolandD Cyber Produksi <contact@csvimproved.com>
 * @copyright   Copyright (C) 2006 - 2017 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://csvimproved.com
 */

defined('_JEXEC') or die;

/**
 * The Form2Content helper class.
 *
 * @package     CSVI
 * @subpackage  Form2Content
 * @since       7.0
 */
class Com_Form2ContentHelperCom_Form2Content
{
	/**
	 * Template helper
	 *
	 * @var    CsviHelperTemplate
	 * @since  7.0
	 */
	protected $template = null;

	/**
	 * Logger helper
	 *
	 * @var    CsviHelperLog
	 * @since  7.0
	 */
	protected $log = null;

	/**
	 * Fields helper
	 *
	 * @var    CsviHelperFields
	 * @since  7.0
	 */
	protected $fields = null;

	/**
	 * Database connector
	 *
	 * @var    JDatabaseDriver
	 * @since  7.0
	 */
	protected $db = null;

	/**
	 * Constructor.
	 *
	 * @param   CsviHelperTemplate  $template  An instance of CsviHelperTemplate.
	 * @param   CsviHelperLog       $log       An instance of CsviHelperLog.
	 * @param   CsviHelperFields    $fields    An instance of CsviHelperFields.
	 * @param   JDatabase           $db        Database connector.
	 *
	 * @since   7.0
	 */
	public function __construct(CsviHelperTemplate $template, CsviHelperLog $log, CsviHelperFields $fields, JDatabase $db)
	{
		$this->template = $template;
		$this->log = $log;
		$this->fields = $fields;
		$this->db = $db;
	}

	/**
	 * Get the category ID based on it's path.
	 *
	 * @param   string  $category_path  The path of the category
	 *
	 * @return  int  The ID of the category.
	 *
	 * @since   7.0
	 */
	public function getCategoryId($category_path)
	{
		$catid = 2;

		if ($category_path)
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('id'))
				->from($this->db->quoteName('#__categories'))
				->where($this->db->quoteName('extension') . ' = ' . $this->db->quote('com_content'))
				->where($this->db->quoteName('path') . ' = ' . $this->db->quote($category_path));
			$this->db->setQuery($query);
			$catid = $this->db->loadResult();
			$this->log->add('Found category id for path ' . $category_path);

			if (!$catid)
			{
				$this->log->add('Category id for path not found so setting to default id ');
				$catid = 2;
			}
		}

		$this->fields->set('catid', $catid);

		return $catid;
	}

	/**
	 * Get the project id, this is necessary for updating existing content.
	 *
	 * @param   string  $title         The project title
	 * @param   bool    $checkMessage  True or False to display title message
	 *
	 * @return  mixed  Int The ID of the article | False if ID has not been found.
	 *
	 * @since   7.0
	 */
	public function getProjectId($title, $checkMessage)
	{
		if (!$title)
		{
			return false;
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__f2c_project'))
			->where($this->db->quoteName('title') . ' = ' . $this->db->quote($title));
		$this->db->setQuery($query);
		$this->log->add('Found project id for title ' . $title);

		$projectId = $this->db->loadResult();

		if (!$projectId && $checkMessage)
		{
			$this->log->add('No project id found for title ' . $title);
			$this->log->addStats('skipped', JText::sprintf('COM_CSVI_CANNOT_FIND_PROJECT_TITLE', $title));

			return false;
		}

		return $projectId;
	}

	/**
	 * Get the field type id.
	 *
	 * @param   string  $name  The field type name
	 *
	 * @return  mixed  Int The ID of the article | False if ID has not been found.
	 *
	 * @since   7.0
	 */
	public function getFieldTypeId($name)
	{
		if (!$name)
		{
			return false;
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__f2c_fieldtype'))
			->where($this->db->quoteName('name') . ' = ' . $this->db->quote($name));
		$this->db->setQuery($query);
		$this->log->add('Found field type id for name' . $name);

		return $this->db->loadColumn();
	}

	/**
	 * Get the field type id.
	 *
	 * @param   string  $name       The field type name
	 * @param   string  $projectId  The project id
	 *
	 * @return  mixed  Int The ID of the article | False if ID has not been found.
	 *
	 * @since   7.0
	 */
	public function getFieldId($name, $projectId)
	{
		if (!$name || !$projectId)
		{
			return false;
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__f2c_projectfields'))
			->where($this->db->quoteName('projectid') . ' = ' . (int) $projectId)
			->where($this->db->quoteName('fieldname') . ' = ' . $this->db->quote($name));
		$this->db->setQuery($query);
		$this->log->add('Found field id for name: ' . $name);

		return $this->db->loadResult();
	}

	/**
	 * Create the correct path from a category path.
	 *
	 * @param   string  $title  The category title to get the path for.
	 *
	 * @return  string  The category path.
	 *
	 * @since   7.0
	 */
	public function convertCategoryPath($title)
	{
		// Turn the category path into aliases
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__categories'))
			->where($this->db->quoteName('title') . ' = ' . $this->db->quote($title), 'OR')
			->where($this->db->quoteName('path') . ' = ' . $this->db->quote($title));
		$this->db->setQuery($query);
		$categoryId = $this->db->loadResult();

		if (!$categoryId)
		{
			return '';
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('p.alias'))
			->from($this->db->quoteName('#__categories', 'n') . ', ' . $this->db->quoteName('#__categories', 'p'))
			->where('n.lft BETWEEN p.lft AND p.rgt')
			->where($this->db->quoteName('n.id') . ' = ' . (int) $categoryId)
			->order($this->db->quoteName('p.lft'));
		$this->db->setQuery($query);

		$segments = $this->db->loadColumn();

		// Make sure to remove the root path if it exists in the list.
		if ($segments[0] == 'root')
		{
			array_shift($segments);
		}

		// Build the path.
		return trim(implode('/', $segments), ' /\\');
	}

/**
	 * Get the f2c article id, this is necessary for updating existing content.
	 *
	 * @param   string  $alias  The alias of the article.
	 *
	 * @return  mixed  Int The ID of the article | False if ID has not been found.
	 *
	 * @since   7.0
	 */
	public function getArticleId($alias)
	{
		$id = $this->fields->get('id');

		if ($id)
		{
			return $id;
		}

		$catid = $this->fields->get('catid');

		if (!$catid)
		{
			$category_path = $this->fields->get('category_path');

			if ($category_path)
			{
				// We have a category path, let's get the ID
				$catid = $this->getCategoryId($this->convertCategoryPath($category_path));

				if (!$catid)
				{
					return false;
				}
			}
		}

		if (!$alias || !$catid)
		{
			return false;
		}

		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__f2c_form'))
			->where($this->db->quoteName('alias') . ' = ' . $this->db->quote($alias))
			->where($this->db->quoteName('catid') . ' = ' . (int) $catid);
		$this->db->setQuery($query);
		$this->log->add('Found article id');

		return $this->db->loadResult();
	}

	/**
	 * Get a list of subcategory IDs for a given parent category ID.
	 *
	 * @param   int  $categoryId  The category ID to get the subcategories for.
	 *
	 * @return  array  List of subcategory IDs.
	 *
	 * @since   7.1.0
	 */
	public function getSubCategoryIds($categoryId)
	{
		$subcats = $this->getChildren($categoryId);

		if (!empty($subcats))
		{
			foreach ($subcats as $subcat)
			{
				$newcats = $this->getSubCategoryIds($subcat);

				$subcats = array_merge($subcats, $newcats);
			}
		}

		return $subcats;

	}

	/**
	 * Get a list of child category if for given category ID.
	 *
	 * @param   int  $parentId  The category id of the parent.
	 *
	 * @return  array  List of subcategory IDs.
	 *
	 * @since   7.1.0
	 */
	private function getChildren($parentId)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__categories'))
			->where($this->db->quoteName('parent_id') . ' = ' . (int) $parentId);
		$this->db->setQuery($query);

		return $this->db->loadColumn();
	}
}
